/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::viscosityModels::HerschelBulkleyDebrisFlow

Description
     Herschel-Bulkley non-Newtonian viscosity model.

SourceFiles
    HerschelBulkleyDebrisFlow.C

\*---------------------------------------------------------------------------*/

#ifndef HerschelBulkleyDebrisFlow_H
#define HerschelBulkleyDebrisFlow_H

#include "viscosityModel.H"
#include "dimensionedScalar.H"
#include "volFields.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace viscosityModels
{

/*---------------------------------------------------------------------------*\
                           Class HerschelBulkleyDebrisFlow Declaration
\*---------------------------------------------------------------------------*/

class HerschelBulkleyDebrisFlow
:
    public viscosityModel
{
    // Private data

        dictionary HerschelBulkleyDebrisFlowCoeffs_;

        
        dimensionedScalar n_;
        
	dimensionedScalar Cv_;
	dimensionedScalar P0v_;
	dimensionedScalar tau00_;
        dimensionedScalar rhoF_;
        dimensionedScalar nu0_;
	dimensionedScalar tau0_;
	dimensionedScalar k_;
	bool setPar_;
        volScalarField nu_;
	
	
	
    // Private Member Functions
	bool setHBParameters();
        //- Calculate and return the laminar viscosity
        tmp<volScalarField> calcNu() const;



public:

    //- Runtime type information
    TypeName("HerschelBulkleyDebrisFlow");


    // Constructors

        //- Construct from components
        HerschelBulkleyDebrisFlow
        (
            const word& name,
            const dictionary& viscosityProperties,
            const volVectorField& U,
            const surfaceScalarField& phi
        );


    //- Destructor
    ~HerschelBulkleyDebrisFlow()
    {}


    // Member Functions

        //- Return the laminar viscosity
        tmp<volScalarField> nu() const
        {
            return nu_;
        }

        //- Return the laminar viscosity for patch
        tmp<scalarField> nu(const label patchi) const
        {
            return nu_.boundaryField()[patchi];
        }
	
        //- Correct the laminar viscosity
        void correct()
        {
            nu_ = calcNu();
        }

        //- Read transportProperties dictionary
        bool read(const dictionary& viscosityProperties);
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace viscosityModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
